#define VERSION "vfl 1.0"

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include "vfl.h"

static char *code;
static int memSize = 256;
static int stackSize = 256;

void portOut(int32_t port, int32_t value) {
	switch (port) {
		case 0:
			putc(value, stdout);
			break;
		case 1:
			printf("%d", value);
			break;
	}
}

int32_t portIn(int32_t port) {
	int32_t value = 0;
	
	switch (port) {
		case 0:
			value = getc(stdin);
			if (feof(stdin)) value = -1;
			break;
		case 1:
			scanf("%d", &value);
			break;
	}
	
	return value;
}

static void printCliUsage() {
	fprintf(stderr, "\
usage:\n\
vfl [options] [file]\n\
\n\
options:\n\
-v: print version\n\
-m [size]: allocate [size] memory cells (default is 256)\n\
-s [size]: allocate [size] cells for each stack (default is 256)\n\
");
}

int main(int argc, char **argv) {
	int curArg = 1;
	while (curArg < argc && argv[curArg][0] == '-') {
		if (!strcmp(argv[curArg], "-v")) {
			printf("%s\n", VERSION);
			return 0;
			
		} else if (!strcmp(argv[curArg], "-m")) {
			curArg++;
			int size = atoi(argv[curArg]);
			
			if (size < 1) {
				fprintf(stderr, "invalid memory size: %d\n", size);
				return 1;
			}
			
			memSize = size;
		
		} else if (!strcmp(argv[curArg], "-s")) {
			curArg++;
			int size = atoi(argv[curArg]);
			
			if (size < 1) {
				fprintf(stderr, "invalid stack size: %d\n", size);
				return 1;
			}
			
			stackSize = size;
		
		} else {
			fprintf(stderr, "unrecognized flag: %s\n", argv[curArg]);
			return 1;
		}
		curArg++;
	}
	
	if (argc < curArg+1) {
		printCliUsage();
		return 1;
	}
	
	char *infile = argv[curArg];
	FILE *file = fopen(infile, "r");
	if (file == NULL) {
		fprintf(stderr, "could not open file; does it exist?\n");
		return 1;
	}
	
	fseek(file, 0L, SEEK_END);
	long size = ftell(file);
	code = malloc(size+1);
	rewind(file);
	fread(code, 1, size, file);
	fclose(file);
	code[size] = '\0';
	
	vfl_init(memSize, stackSize);
	vfl_run(code);
	
	return 0;
}
